package com.example.GPSCam;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class DBObject extends SQLiteOpenHelper {

	// Database Version
	private static final int DATABASE_VERSION = 1;

	// Database Name
	private static final String DATABASE_NAME = "potatoMemNap_DB";

	// Contacts table name
	private static final String TABLE_IMAGEINFO = "info";

	// Contacts Table Columns names
	private static final String KEY_ID = "id";
	private static final String KEY_PATH = "path";
	private static final String KEY_LAT = "lat";
	private static final String KEY_LNG = "lng";
	private static final String KEY_DESCRIPTION = "description";

	// Constructor for database object
	public DBObject(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	// called when a DBObject is created
	@Override
	public void onCreate(SQLiteDatabase db) {
		// create the table
		String CREATE_IMAGEINFO_TABLE = "CREATE TABLE " + TABLE_IMAGEINFO + "("
				+ KEY_ID + " INTEGER PRIMARY KEY," + KEY_PATH + " TEXT,"
				+ KEY_LAT + " TEXT," + KEY_LNG + " TEXT," + KEY_DESCRIPTION
				+ " TEXT" + ")";
		db.execSQL(CREATE_IMAGEINFO_TABLE);
	}

	// used when structure of DB is changed
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// Drop older table if existed
		db.execSQL("DROP TABLE IF EXISTS " + TABLE_IMAGEINFO);

		// Create tables again
		onCreate(db);
	}

	// add new image
	public void addImage(ImageObject io) {
		// open DB for reading and writting
		SQLiteDatabase db = this.getWritableDatabase();
		Log.d("in dbo addimage",
				"path = " + io.getPath().getPath() + " lat = " + io.getLat()
						+ " lng = " + io.getLng() + " desc = "
						+ io.getDescription());

		// informations to be inserted: path,lat,lng,desc
		ContentValues values = new ContentValues();
		values.put(KEY_PATH, io.getPath().getPath());
		values.put(KEY_LAT, io.getLat());
		values.put(KEY_LNG, io.getLng());
		values.put(KEY_DESCRIPTION, io.getDescription());

		// insert
		db.insert(TABLE_IMAGEINFO, null, values);
		db.close(); // Closing database connection
	}

	// retrieve single image
	public ImageObject getImageInfo(String path) {
		SQLiteDatabase db = this.getReadableDatabase();

		// db.query: 1st para = table name, 2nd param = info to retrive, 3rd
		// param = "where" clause in sql which "?" is replaced by the 4th param,
		// followed by groupby, orderby, having and limit
		Cursor cursor = db.query(TABLE_IMAGEINFO, new String[] { KEY_PATH,
				KEY_LAT, KEY_LNG, KEY_DESCRIPTION }, KEY_PATH + "=?",
				new String[] { String.valueOf(path) }, null, null, null, null);
		//if there are >=1 records, go to the 1st record
		if (cursor != null)
			cursor.moveToFirst();

		File imageFilePath = new File(cursor.getString(0));
		Log.d("retiving", "retrive imagepath = " + imageFilePath.getPath());
		Log.d("retiving", "retrive lat = " + cursor.getDouble(1));
		Log.d("retiving", "retrive lng = " + cursor.getDouble(2));
		Log.d("retiving", "retrive desc = " + cursor.getString(3));
		
		//retrive the info and return as an image object
		ImageObject io = new ImageObject(imageFilePath, cursor.getDouble(1),
				cursor.getDouble(2), cursor.getString(3));

		return io;
	}

	//retrieve all image info
	public List<ImageObject> getAllContacts() {
		List<ImageObject> imageList = new ArrayList<ImageObject>();
		//sql query to select a everything from table
		String selectQuery = "SELECT  * FROM " + TABLE_IMAGEINFO;

		SQLiteDatabase db = this.getWritableDatabase();
		//rawquery takes in sql statement unlike query which has a structured manner
		Cursor cursor = db.rawQuery(selectQuery, null);

		//loop through all rows and adding to list
		if (cursor.moveToFirst()) {
			do {
				ImageObject io = new ImageObject();
				File imageFilePath = new File(cursor.getString(1));
				io.setPath(imageFilePath);
				io.setLat(cursor.getDouble(2));
				io.setLng(cursor.getDouble(3));
				io.setDescription(cursor.getString(4));
				//add item to list
				imageList.add(io);
			} while (cursor.moveToNext());
		}
		return imageList;
	}

	//get number of items
	public int getImageCount() {
		String countQuery = "SELECT  * FROM " + TABLE_IMAGEINFO;
		SQLiteDatabase db = this.getReadableDatabase();
		Cursor cursor = db.rawQuery(countQuery, null);
		int imageCount = cursor.getCount();
		cursor.close();

		return imageCount;
	}

	//update
	public int updateContact(ImageObject io) {
		SQLiteDatabase db = this.getWritableDatabase();

		//columns that is to be updated, here, only desc
		ContentValues values = new ContentValues();
		values.put(KEY_DESCRIPTION, io.getDescription());

		//update row
		return db.update(TABLE_IMAGEINFO, values, KEY_PATH + " = ?",
				new String[] { String.valueOf(io.getPath()) });
	}

	//delete an image
	public void deleteContact(ImageObject io) {
		SQLiteDatabase db = this.getWritableDatabase();
		db.delete(TABLE_IMAGEINFO, KEY_PATH + " = ?",
				new String[] { String.valueOf(io.getPath()) });
		db.close();
	}

}
