package com.example.GPSCam;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.media.ExifInterface;
import android.media.MediaScannerConnection;
import android.media.MediaScannerConnection.MediaScannerConnectionClient;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.text.format.Time;
import android.util.Log;
import android.widget.Toast;

public class ImageObject implements LocationListener,
		MediaScannerConnectionClient {

	File path;
	double lat;
	double lng;
	String description;

	public ImageObject() {

	}

	public ImageObject(File path, double lat, double lng, String description) {
		this.path = path;
		this.lat = lat;
		this.lng = lng;
		this.description = description;
	}

	public double getLat() {
		return lat;
	}

	public void setLat(double lat) {
		this.lat = lat;
	}

	public double getLng() {
		return lng;
	}

	public void setLng(double lng) {
		this.lng = lng;
	}

	public File getPath() {
		return path;
	}

	public void setPath(File path) {
		this.path = path;
	}
	
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	// this method is used to define the path of the photo taken
	protected File setUpImageFile(Context context, int GPSValid, ImageObject io) {
		MediaScannerConnection msc;
		// establish mediascanner connection
		msc = new MediaScannerConnection(context, this);
		msc.connect();

		String fileName = "";
		File pictureDir = null;

		// determine the directory of the image depending on whether GPS signal
		// available and time of the day
		if (GPSValid == 0) {
			pictureDir = new File(Environment.getExternalStorageDirectory()
					.toString()
					+ File.separator
					+ "PotatoMemNap"
					+ File.separator + "GPSValid");
		} else if (GPSValid == 1) {
			// Create Time object
			Time now = new Time();
			// get the current time
			now.setToNow();
			// get the hour of the current time
			int hourNow = now.hour;

			if (hourNow >= 6 && hourNow <= 11) {
				// morning
				pictureDir = new File(Environment.getExternalStorageDirectory()
						.toString()
						+ File.separator
						+ "PotatoMemNap"
						+ File.separator
						+ "GPSInvalid"
						+ File.separator
						+ "Morning");
			} else if (hourNow >= 12 && hourNow <= 17) {
				// afternoon
				Log.d("check error", "im here before specify");
				pictureDir = new File(Environment.getExternalStorageDirectory()
						.toString()
						+ File.separator
						+ "PotatoMemNap"
						+ File.separator
						+ "GPSInvalid"
						+ File.separator
						+ "Afternoon");
				Log.d("check error", "im here after specify");
			} else if (hourNow >= 18 && hourNow <= 23) {
				// night
				pictureDir = new File(Environment.getExternalStorageDirectory()
						.toString()
						+ File.separator
						+ "PotatoMemNap"
						+ File.separator
						+ "GPSInvalid"
						+ File.separator
						+ "Night");
			} else if (hourNow >= 0 && hourNow <= 5) {
				// latenight
				pictureDir = new File(Environment.getExternalStorageDirectory()
						.toString()
						+ File.separator
						+ "PotatoMemNap"
						+ File.separator
						+ "GPSInvalid"
						+ File.separator
						+ "Late_night");
			}
		}

		// check if unable to create folder if it does not exist already
		if (!pictureDir.exists() && !pictureDir.mkdirs()) {
			Log.d("directory", "Can't create directory to save image.");
			Toast.makeText(context, "Can't create directory to save image.",
					Toast.LENGTH_LONG).show();
		}

		// construct the name of the picture taken
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyymmddhhmmss");
		String date = dateFormat.format(new Date());
		String photoFile = "Picture_" + date + ".jpg";

		// resulting name of image i.e.
		// sdcard\cameraDemo\Picture_xxxxxxxxxxxxxx.jpg
		Log.d("in setupfile check validgps", "" + GPSValid);
		fileName = pictureDir.getPath() + File.separator + photoFile;
		File pictureFile = new File(fileName);

		Log.d("in setupfile check validgps and path", "GPSvalid: " + GPSValid
				+ "  path: " + pictureFile.getPath());
		io.setPath(pictureFile);
		return pictureFile;
	}

	// this method is used to trigger media scanner to allow photos taken to be
	// read
	public void triggerMediaScanner(File pictureFile, Context context) {
		Intent mediaScanIntent = new Intent(
				Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
		File f = new File(pictureFile.getAbsolutePath());
		Uri contentUri = Uri.fromFile(f);
		mediaScanIntent.setData(contentUri);
		context.sendBroadcast(mediaScanIntent);
	}

	// save info to EXIF
	protected void saveToEXIF(File path, String latDegree, String lngDegree,
			String latMin, String lngMin, String latSec, String lngSec,
			String sLatR, String sLngR) {

		try {
			// instantiate exifinterface
			Log.d("error check in exif", "error check in exif");
			ExifInterface exifInterface = new ExifInterface(path.toString());
			Log.d("error check after instant exif",
					"error check after instant exif, path = " + path);
			String fLat = "";
			String fLng = "";
			String fLatR = "";
			String fLngR = "";

			// values for the various EXIF interface
			fLat = latDegree + "/1," + latMin + "/1," + latSec + "/1";
			fLng = lngDegree + "/1," + lngMin + "/1," + lngSec + "/1";
			fLatR = sLatR;
			fLngR = sLngR;

			// set the attributes for lat long
			exifInterface.setAttribute(ExifInterface.TAG_GPS_LATITUDE, fLat);

			exifInterface.setAttribute(ExifInterface.TAG_GPS_LONGITUDE, fLng);

			exifInterface.setAttribute(ExifInterface.TAG_GPS_LATITUDE_REF,
					fLatR);

			exifInterface.setAttribute(ExifInterface.TAG_GPS_LONGITUDE_REF,
					fLngR);

			// save the attributes
			exifInterface.saveAttributes();
			// this shows how to retrive info from exif
			Log.d("EXIF DATA",
					"lat: "
							+ exifInterface
									.getAttribute(ExifInterface.TAG_GPS_LONGITUDE)
							+ " lng: "
							+ exifInterface
									.getAttribute(ExifInterface.TAG_GPS_LONGITUDE));
		} catch (IOException e) {
			Log.d("Error", e.getMessage());
			return;
		}
	}

	public void onMediaScannerConnected() {
		Log.d("scanner connected", "scanner connected");
	}

	public void onScanCompleted(String path, Uri uri) {
		Log.d("scanned comeplete", " scan complete");
	}

	public void onLocationChanged(Location location) {
		// TODO Auto-generated method stub

	}

	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub

	}

	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub

	}

	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub

	}

}
